<!DOCTYPE html>
<html>
<head>
    {% include 'partials/head.html' %}
    <link rel="stylesheet" href="{{ url_for('static', filename='css/base.css') }}">
    <link rel="stylesheet" href="{{ url_for('static', filename='css/layout.css') }}">
    <link rel="stylesheet" href="{{ url_for('static', filename='css/components.css') }}">
</head>
<body>
    <div class="page-container">
        {% include 'partials/banner.html' %}
        <div class="main-body">
                {% block content %}{% endblock %}
        </div>
        {% include 'partials/footer.html' %}
    </div>


    <script src="https://unpkg.com/vue@3"></script>
    <script src="https://cdn.jsdelivr.net/npm/markdown-it/dist/markdown-it.min.js"></script>
    <script src="{{ url_for('static', filename='js/common.js') }}"></script>
    
    {% block scripts %}{% endblock %}

</body>
</html>
