html, body {
    height: 100%;
    margin: 0;
}

.page-container {
    display: flex;
    flex-direction: column;
    min-height: 100vh;
}

/* Top header with logos and user info */
.header-banner {
    display: flex;
    justify-content: space-between;  /* Keep uni logo and profile on opposite ends */
    align-items: center;  /* Vertically center all direct children */
    padding: 0 15rem;  /* Horizontal padding to align with main body */
}

.header-banner img.logo  {
    width: 240px;  
    object-fit: contain;
}

.header-banner img.profile {
    height: 80px;
    object-fit: contain;
}
.banner-right {
    display: flex;
    align-items: center;
    gap: 1.5em;
}
.user-banner {
    display: flex;
    align-items: center;
    gap: 1em;
}

.inline-divider {
    height: 20px;
    width: auto;
    vertical-align: middle;  /* Aligns divider SVG images with text */
}

/*main-body holds the main page (under top-menu) */
.main-body {
    flex: 1;
    display: flex;
    flex-direction: column;
    padding: 0 15rem;  /* optional: adds left/right breathing room */
}

.content-row {
    display: flex;
    flex: 1;
}

.content-row {
    display: flex;
    flex: 1;
    align-items: flex-start;
    padding: 0;
    margin: 0;
}

.left-menu {
    width: 15rem; /* Controls the width in the layout */
    margin-right: 10px;
    padding: 0px;
    box-sizing: border-box;
    min-height: 20em;
}

/* main-content is the quiz/kb/assignment currently working on */
.main-content {
    flex: 1;
    padding: 0px;
    box-sizing: border-box;
}

/*Some useful elements:*/

/* Alert notifications */
.overlay {
    position: fixed;
    top: 0;
    left: 0;
    width: 100%;
    height: 100%;
    display: none;
    justify-content: center;
    align-items: center;
    z-index: 1000;
}

/* row of buttons */
.button-row {
    display: flex;
    gap: 10px;
    justify-content: start;
    flex-wrap: wrap;
    align-items: center;
}