body { 
	font-family: 'Open Sans', sans-serif;
	font-size: 14px;
}

h1, h2, h3, h4, h5, h6 {
    font-weight: 600; /* Semi bold */
    padding-top: 5px; 
    padding-bottom: 5px; 
    margin: 0; /* Optional: reset margins if needed */
}